//::///////////////////////////////////////////////
//:: Default On Enter for Module
//:: x4_mod_def_enter
//:: Copyright (c) 2014 Bioware Corp.
//:://////////////////////////////////////////////
/*
     This script adds the horse menus to the PCs
     and binds CD-key to player login to avoid
     other players logging on accounts others than
     their own.
*/
//:://////////////////////////////////////////////
//:: Created By: Deva B. Winblood
//:: Created On: Dec 30th, 2007
//:: Last Update: April 21th, 2008
//:://////////////////////////////////////////////

#include "x3_inc_horse"
#include "x4_inc_secure"
#include "quest_inc"

void Transport(object oPC, location lDest, string sWaypoint)
{
    if (!GetIsObjectValid(oPC))
        return;
    if (!GetIsObjectValid(GetArea(oPC)))
    {
        DelayCommand(1.0, Transport(oPC, lDest, sWaypoint));
    }
    else if(GetTag(GetArea(oPC)) != GetTag(GetArea(GetWaypointByTag(sWaypoint))))
    {
        AssignCommand(oPC, JumpToLocation(lDest));
    }
}

void RemoveAllItems(object oCreature, string sTag="")
{
    object oItem = GetFirstItemInInventory(oCreature);
    while ( oItem != OBJECT_INVALID )
    {
        if ( "" == sTag  ||  GetTag(oItem) == sTag )
            DestroyObject(oItem);
        oItem = GetNextItemInInventory(oCreature);
    }
}

void main()
{
    object oPC=GetEnteringObject();

    AddJournalQuestEntry("xprules", 1, oPC, FALSE, FALSE, FALSE);
    AddJournalQuestEntry("lvl_adj", 1, oPC, FALSE, FALSE, FALSE);

    DeleteLocalInt(oPC, "TalkedAdv");
    DeleteLocalInt(oPC, "Duel");

    /* verify CD keys and double logins to stop hackers if the module switch is active */
    if (GetLocalString(GetModule(), "X4_SECURE_LOGIN") == "TRUE")
    {
    if (VerifyPlayernameAgainstCDKey(oPC))
        {
        if (GetIsObjectValid(oPC))
            BootPC(oPC);
        return;
        }
    }

    // Database initialization
    object oModule = GetModule();
    string sDB = GetLocalString(oModule, "DB");
    string sAuth = DBAccountSpecificVariable(oPC, "Auth");
    if (GetCampaignInt(sDB, "db_init") == 0)
    {
    SetCampaignInt(sDB, "db_init", 1);
    if(GetLocalString(oModule, "MODE") == "LOCAL") SetCampaignInt(sDB, sAuth, 1);
    SetCampaignString(sDB, "difficulty_death", "ADVENTURER");
    SetCampaignString(sDB, "difficulty_explo", "ADVENTURER");
    SetCampaignString(sDB, "difficulty_enemy", "ADVENTURER");
    SetCampaignString(sDB, "difficulty_adven", "ADVENTURER");
    SetCampaignString(sDB, "difficulty_commu", "ADVENTURER");
    SetLocalString(oModule, "mDiffDeath", "ADVENTURER");
    SetLocalString(oModule, "mDiffExplo", "ADVENTURER");
    SetLocalString(oModule, "mDiffEnemy", "ADVENTURER");
    SetLocalString(oModule, "mDiffAdven", "ADVENTURER");
    SetLocalString(oModule, "mDiffCommu", "ADVENTURER");
    GenerateAdventurerNPC(1);
    GenerateAdventurerNPC(2);
    GenerateAdventurerNPC(3);
    string sEvent;
    int i;
    //starting events
    for (i=1; i<=3; i++)
    {
    switch (Random(14))
        {
            case 0: sEvent = "island"; break; //some ship encountered an unknown island
            case 1: sEvent = "north"; break; //another explorer has gone to the north and hasn't returned
            case 2: sEvent = "drow"; break; //elves captured a drow spy in Serine Derthore
            case 3: sEvent = "nuvaris"; break; //the city of Nuvaris has very dynamic politics, a new ruler might emerge soon
            case 4: sEvent = "underdark"; break; //more drow and duergar sightings on the surface, possibility of invasion
            case 5: sEvent = "tomb"; break; //explorer discovered a tomb in Shahtir desert and was ripped apart by a mummy
            case 6: sEvent = "necromancer"; break; //sightings of undead in Green Coast, a necromancer might be gathering troops
            case 7: sEvent = "ghoul"; break; //a travelling merchant was assaulted by a ghoul while traversing Corrupted Wetlands
            case 8: sEvent = "artifact"; break; //there's an ancient artifact hidden somewhere that's said to be powerful beyond imagination
            case 9: sEvent = "masked"; break; //a masked figure has been seen throughout the realm, wears a skull mask, might be a lich
            case 10: sEvent = "dragons"; break; //there was a battle of dragons above the Grey Peaks, not known which dragon won
            case 11: sEvent = "barbarian"; break; //barbarian started a tavern brawl in the town, hardly unexpected
            case 12: sEvent = "friends"; break; //there was an adventurer party visiting with a dwarf and an elf who seemed friends, surprise
            case 13: sEvent = "orcs"; break; //orcs spotted near the town, which is concerning
            case 14: sEvent = "wizard"; break; //a wizard tried to time travel to the past and disappeared completely
            case 15: sEvent = "nymph"; break; //a friend travelling through the forest heard whispers encouraging him to go off the road; he suspects it was a nymph
            case 16: sEvent = "caravan"; break; //a caravan has been attacked, but was guarded
        }
    SetCampaignString(sDB, IntToString(i)+"_EVENT", sEvent);
    }

    }
    //Send a warning if the module detects that a save file has been loaded
    if (GetLocalInt(oModule, "SAVELOADED") == TRUE)
    DelayCommand(5.0, SendMessageToPC(oPC, ColorString("WARNING:", 255, 1, 1)+ColorString(" It looks like you have loaded a save file. Ancordia is not meant to be saved and loaded. You are advised to export your character and launch the module as a new game, choosing your exported character in the process. If you really want to play on loaded save files, use them only to continue previously halted sessions, NOT as a means of going back to an earlier moment of the session.", 255, 255, 255)));

    // CD-key verification
    string sKey = GetPCPublicCDKey(oPC, TRUE);
    string sKeySet = DBAccountSpecificVariable(oPC, "KeySet");
    string sCDKey = DBAccountSpecificVariable(oPC, "CDKey");
    if (GetCampaignInt(sDB, sKeySet) == 0)
        {
        SetCampaignString(sDB, sCDKey, sKey);
        SetCampaignInt(sDB, sKeySet, 1);
        }
    else if (sKey != GetCampaignString(sDB, sCDKey))
    {
        BootPC(oPC);
        if (!GetIsSinglePlayer()) return;
    }
    if (sKey == "Q7RREKF3") //boot PCs using default GOG.com key in multiplayer games and warn them in single-player games
    {
        DelayCommand(5.0, SendMessageToPC(oPC, ColorString("WARNING:", 255, 1, 1)+ColorString(" You are currently using the default GOG.com cd-key. You can ignore this warning if you don't plan on playing the game in multiplayer, but otherwise you should replace the keys found in 'nwncdkey.ini' file with your unique ones.", 255, 255, 255)));
        BootPC(oPC);
        if (!GetIsSinglePlayer()) return;
    }

    //Boot the PC if their login is longer than 12 digits (or inform them of this if booting is impossible
    string sLogin = GetPCPlayerName(oPC);
    if (GetStringLength(sLogin) > 12)
    {
        SendMessageToPC(oPC, "WARNING: Your player name (login) is over 12 digits long! This can mess the game badly. Please consider changing your player name in the multiplayer login screen to something shorter and creating a new character.");
        BootPC(oPC);
        if (!GetIsSinglePlayer()) return;
    }

    //Save the character database name as string on the PC (for use in the OnClientLeave event)
    string sPlayerDB = CharacterDB(oPC);
    SetLocalString(oPC, "CharDatabase", sPlayerDB);


    //Teleport entering PCs
    if (GetIsPC(oPC) == TRUE)
    {
        //If the PC was dead, let's make him/her dead again! No escape, mwahahaa!
       /*if (GetCampaignInt(CharacterDB(oPC), "PC_DEAD") == TRUE)
        {
            Transport(oPC, GetLocation(GetWaypointByTag("PC_START")), "PC_START");
        }
        else */

        if (GetCampaignInt(CharacterDB(oPC), "MILES") != 0) //we need to check if the PC should end up in the return area
        {
            string sRegion = GetCampaignString(CharacterDB(oPC), "REGION");
            sRegion = "REG_"+sRegion+"_RETURN";
            SetLocalInt(oPC, "Miles", GetCampaignInt(CharacterDB(oPC), "MILES"));
            Transport(oPC, GetLocation(GetWaypointByTag(sRegion)), sRegion);
        }
        else if (GetLocalInt(GetModule(), "ANC_SPAWN") == 1)
        {
            string sSpawn = GetCampaignString(CharacterDB(oPC), "TOWN_SPAWN");
            if (sSpawn == "") sSpawn = "WP_AREA_GREENTREE";
            object oWP = GetWaypointByTag(sSpawn);
            Transport(oPC, GetLocation(oWP), sSpawn);
        }
    }

    //Cancel any polymorphs the player might have
    RemoveSpecificEffect(EFFECT_TYPE_POLYMORPH, oPC);

    //Add journal entries
    if (GetCampaignInt(CharacterDB(oPC), "STORY_1FINISH") == TRUE) AddJournalQuestEntry("story_quest01", 2, oPC, FALSE);
    else if (GetCampaignInt(CharacterDB(oPC), "STORY_1ACCEPT") == TRUE) AddJournalQuestEntry("story_quest01", 1, oPC, FALSE);

    //Assign the PC a unique string value from 01 to 20, to make them use unique set of tokens and avoid invalid quest data display in the journal
    if (GetIsSinglePlayer() == TRUE) SetLocalString(oPC, "QuestToken", "01");
    else
    {
        int i = 1;
        string sToken = "01";
        while (TRUE)
        {
            if (!GetIsObjectValid(GetLocalObject(oModule, "Player"+sToken)))
            {
                SetLocalObject(oModule, "Player"+sToken, oPC);
                SetLocalString(oPC, "QuestToken", sToken);
                break;
            }
            i++;
            if (i > 20) break;
            sToken = IntToString(i);
            if (i < 10) sToken = "0"+sToken;
        }
    }

    //Add random quest journal entry the PC should have
    RemoveJournalQuestEntry("random_quest01", oPC, FALSE);
    RemoveJournalQuestEntry("random_quest02", oPC, FALSE);
    RemoveJournalQuestEntry("random_quest03", oPC, FALSE);
    RemoveJournalQuestEntry("random_quest04", oPC, FALSE);
    RemoveJournalQuestEntry("random_quest05", oPC, FALSE);
    RemoveJournalQuestEntry("random_quest06", oPC, FALSE);
    RemoveJournalQuestEntry("random_quest07", oPC, FALSE);
    RemoveJournalQuestEntry("random_quest08", oPC, FALSE);
    RemoveJournalQuestEntry("random_quest09", oPC, FALSE);
    RemoveJournalQuestEntry("random_quest10", oPC, FALSE);
    RemoveJournalQuestEntry("random_quest11", oPC, FALSE);
    RemoveJournalQuestEntry("random_quest12", oPC, FALSE);
    RemoveJournalQuestEntry("random_quest13", oPC, FALSE);
    RemoveJournalQuestEntry("random_quest14", oPC, FALSE);
    RemoveJournalQuestEntry("random_quest15", oPC, FALSE);
    RemoveJournalQuestEntry("random_quest16", oPC, FALSE);
    RemoveJournalQuestEntry("random_quest17", oPC, FALSE);
    RemoveJournalQuestEntry("random_quest18", oPC, FALSE);
    RemoveJournalQuestEntry("random_quest19", oPC, FALSE);
    RemoveJournalQuestEntry("random_quest20", oPC, FALSE);
    string sCharDB = CharacterDB(oPC);
    int nQuestEntry = GetCampaignInt(sCharDB, "QUEST_JOURNAL");
    if (nQuestEntry != 0)
        {
        SetQuestTokens(oPC);
        AddJournalQuestEntry("random_quest"+GetLocalString(oPC, "QuestToken"), nQuestEntry, oPC, FALSE, FALSE, TRUE);
        }
    else RemoveAllItems(oPC, "quest_proof");

    ExecuteScript("x3_mod_pre_enter",OBJECT_SELF); // Override for other skin systems
    if ((GetIsPC(oPC)||GetIsDM(oPC))&&!GetHasFeat(FEAT_HORSE_MENU,oPC))
    { // add horse menu
        HorseAddHorseMenu(oPC);
        if (GetLocalInt(GetModule(),"X3_ENABLE_MOUNT_DB"))
        { // restore PC horse status from database
            DelayCommand(2.0,HorseReloadFromDatabase(oPC,X3_HORSE_DATABASE));
        } // restore PC horse status from database
    } // add horse menu
    if (GetIsPC(oPC))
    { // more details
        // restore appearance in case you export your character in mounted form, etc.
        if (!GetSkinInt(oPC,"bX3_IS_MOUNTED")) HorseIfNotDefaultAppearanceChange(oPC);
        // pre-cache horse animations for player as attaching a tail to the model
        HorsePreloadAnimations(oPC);
        DelayCommand(3.0,HorseRestoreHenchmenLocations(oPC));
    } // more details
}
