//::///////////////////////////////////////////////
//:: Name x2_def_spawn
//:: Copyright (c) 2001 Bioware Corp.
//:://////////////////////////////////////////////
/*
    Default On Spawn script


    2003-07-28: Georg Zoeller:

    If you set a ninteger on the creature named
    "X2_USERDEFINED_ONSPAWN_EVENTS"
    The creature will fire a pre and a post-spawn
    event on itself, depending on the value of that
    variable
    1 - Fire Userdefined Event 1510 (pre spawn)
    2 - Fire Userdefined Event 1511 (post spawn)
    3 - Fire both events

    2007-12-31: Deva Winblood
    Modified to look for X3_HORSE_OWNER_TAG and if
    it is defined look for an NPC with that tag
    nearby or in the module (checks near first).
    It will make that NPC this horse's master.

*/
//:://////////////////////////////////////////////
//:: Created By: Keith Warner, Georg Zoeller
//:: Created On: June 11/03
//:://////////////////////////////////////////////

const int EVENT_USER_DEFINED_PRESPAWN = 1510;
const int EVENT_USER_DEFINED_POSTSPAWN = 1511;


#include "x2_inc_switches"
#include "x4_inc_functions"
#include "x0_i0_spawncond"
void main()
{
    string sTag;
    object oNPC;
    // User defined OnSpawn event requested?
    int nSpecEvent = GetLocalInt(OBJECT_SELF,"X2_USERDEFINED_ONSPAWN_EVENTS");


    SetLocalInt(OBJECT_SELF, "HenchInv", FALSE);

    // Pre Spawn Event requested
    if (nSpecEvent == 1  || nSpecEvent == 3  )
    {
    SignalEvent(OBJECT_SELF,EventUserDefined(EVENT_USER_DEFINED_PRESPAWN ));
    }

    sTag=GetLocalString(OBJECT_SELF,"X3_HORSE_OWNER_TAG");
    if (GetStringLength(sTag)>0)
    { // look for master
        oNPC=GetNearestObjectByTag(sTag);
        if (GetIsObjectValid(oNPC)&&GetObjectType(oNPC)==OBJECT_TYPE_CREATURE)
        { // master found
            AddHenchman(oNPC);
        } // master found
        else
        { // look in module
            oNPC=GetObjectByTag(sTag);
            if (GetIsObjectValid(oNPC)&&GetObjectType(oNPC)==OBJECT_TYPE_CREATURE)
            { // master found
                AddHenchman(oNPC);
            } // master found
            else
            { // master does not exist - remove X3_HORSE_OWNER_TAG
                DeleteLocalString(OBJECT_SELF,"X3_HORSE_OWNER_TAG");
            } // master does not exist - remove X3_HORSE_OWNER_TAG
        } // look in module
    } // look for master

    /*  Fix for the new golems to reduce their number of attacks */

    int nNumber = GetLocalInt(OBJECT_SELF,CREATURE_VAR_NUMBER_OF_ATTACKS);
    if (nNumber >0 )
    {
        SetBaseAttackBonus(nNumber);
    }

    // Execute default OnSpawn script.
    ExecuteScript("nw_c2_default9", OBJECT_SELF);


    string sDB = GetLocalString(GetArea(OBJECT_SELF), "sDB");
    if (sDB != "")
    {
    int nHead = GetCampaignInt(sDB, "QUEST_CLI_HEAD");
    int nPheno = GetCampaignInt(sDB, "QUEST_CLI_PHENO");
    int nHair = GetCampaignInt(sDB, "QUEST_CLI_HAIR");
    int nSkin = GetCampaignInt(sDB, "QUEST_CLI_SKIN");
    string sCloth = GetCampaignString(sDB, "QUEST_CLI_CLOTH");
    int nLaw = GetCampaignInt(sDB, "QUEST_CLI_LAW");
    int nGood = GetCampaignInt(sDB, "QUEST_CLI_GOOD");

    SetColor(OBJECT_SELF, COLOR_CHANNEL_HAIR, nHair);
    SetColor(OBJECT_SELF, COLOR_CHANNEL_SKIN, nSkin);
    SetPhenoType(nPheno, OBJECT_SELF);
    SetCreatureBodyPart(CREATURE_PART_HEAD, nHead, OBJECT_SELF);

    AdjustAlignment(OBJECT_SELF, ALIGNMENT_CHAOTIC, 100, FALSE);
    AdjustAlignment(OBJECT_SELF, ALIGNMENT_EVIL, 100, FALSE);
    AdjustAlignment(OBJECT_SELF, ALIGNMENT_LAWFUL, nLaw, FALSE);
    AdjustAlignment(OBJECT_SELF, ALIGNMENT_GOOD, nGood, FALSE);

    object oCloth = CreateItemOnObject(sCloth, OBJECT_SELF);
    ActionEquipItem(oCloth, INVENTORY_SLOT_CHEST);
    DeleteLocalString(GetArea(OBJECT_SELF), "sDB");
    }
    else
    {
    object oClothes;
    if(GetLocalInt(GetArea(OBJECT_SELF), "Nobles") == 1)
    {
        switch (Random(9))
        {
        case 0: oClothes = CreateItemOnObject("NW_CLOTH001"); break;
        case 1: oClothes = CreateItemOnObject("NW_CLOTH022"); break;
        case 2: oClothes = CreateItemOnObject("NW_CLOTH006"); break;
        case 3: oClothes = CreateItemOnObject("NW_CLOTH024"); break;
        case 4: oClothes = CreateItemOnObject("NW_CLOTH013"); break;
        case 5: oClothes = CreateItemOnObject("NW_CLOTH003"); break;
        case 6: oClothes = CreateItemOnObject("NW_CLOTH002"); break;
        case 7: oClothes = CreateItemOnObject("NW_CLOTH028"); break;
        case 8: oClothes = CreateItemOnObject("NW_CLOTH018"); break;
        }
    }
    else
    {
        switch (Random(5))
        {
        case 0: oClothes = CreateItemOnObject("NW_CLOTH001"); break;
        case 1: oClothes = CreateItemOnObject("NW_CLOTH022"); break;
        case 2: oClothes = CreateItemOnObject("NW_CLOTH006"); break;
        case 3: oClothes = CreateItemOnObject("NW_CLOTH024"); break;
        case 4: oClothes = CreateItemOnObject("NW_CLOTH013"); break;
        }
    }
    SetDroppableFlag(oClothes, FALSE);
    SetPickpocketableFlag(oClothes, FALSE);
    ClearAllActions();
    EquipItem(OBJECT_SELF, oClothes);
    //Randomize appearance; starting with hair color
    switch(d6())
        {
        case 1: SetColor(OBJECT_SELF, COLOR_CHANNEL_HAIR, 11);break;
        case 2: SetColor(OBJECT_SELF, COLOR_CHANNEL_HAIR, 2);break;
        case 3: SetColor(OBJECT_SELF, COLOR_CHANNEL_HAIR, 135);break;
        case 4: SetColor(OBJECT_SELF, COLOR_CHANNEL_HAIR, 16);break;
        case 5: SetColor(OBJECT_SELF, COLOR_CHANNEL_HAIR, 7);break;
        case 6: SetColor(OBJECT_SELF, COLOR_CHANNEL_HAIR, 15);break;
        }

    //Randomize phenotype; elves and half-elves should be fat veeery rarely
    if ( GetRacialType(OBJECT_SELF) == RACIAL_TYPE_ELF  ||
         GetRacialType(OBJECT_SELF) == RACIAL_TYPE_HALFELF )
    {
    SetLocalInt(OBJECT_SELF, "Chance", 3);
    }
    else
    {
    SetLocalInt(OBJECT_SELF, "Chance", 15);
    }

    if ( Random(100) < GetLocalInt(OBJECT_SELF, "Chance"))
    {
    SetPhenoType(PHENOTYPE_BIG, OBJECT_SELF);
    }
    else
    {
    SetPhenoType(PHENOTYPE_NORMAL, OBJECT_SELF);
    }

    //Randomize skin color; only for humans, elves and dwarves (in case of elves and dwarves, allow them to be drow and duergar, respectively)
    if ( GetRacialType(OBJECT_SELF) == RACIAL_TYPE_HUMAN )
    {
        if ( Random(100) < 2 && GetLocalInt(GetArea(OBJECT_SELF), "NoBlacks") == 0)
        {
        SetColor(OBJECT_SELF, COLOR_CHANNEL_SKIN, 7);
        }
        else
        {
        SetColor(OBJECT_SELF, COLOR_CHANNEL_SKIN, 2);
        }
    }


    /*if ( GetRacialType(OBJECT_SELF) == RACIAL_TYPE_ELF )
    {
        if ( Random(100) < 10)
        {
        SetColor(OBJECT_SELF, COLOR_CHANNEL_SKIN, 134);
        SetLocalString(OBJECT_SELF, "subrace", "drow");
        }
        else
        {
        SetColor(OBJECT_SELF, COLOR_CHANNEL_SKIN, 1);
        }
    }

    if ( GetRacialType(OBJECT_SELF) == RACIAL_TYPE_DWARF )
    {
        if ( Random(100) < 10)
        {
        SetColor(OBJECT_SELF, COLOR_CHANNEL_SKIN, 42);
        SetLocalString(OBJECT_SELF, "subrace", "duergar");
        }
        else
        {
        SetColor(OBJECT_SELF, COLOR_CHANNEL_SKIN, 2);
        }
    }  */

    //Randomize head; choose any head from the default ones numbered 1 to 10
    SetCreatureBodyPart(CREATURE_PART_HEAD, d10(), OBJECT_SELF);

    if(GetLocalInt(OBJECT_SELF, "ToSit") == TRUE)
    {
    ActionSit(GetNearestObjectByTag("Chair"));
    SetLocalInt(OBJECT_SELF,"sit",1);
    }
    else if (GetCreatureFlag(OBJECT_SELF, CREATURE_VAR_USE_SPAWN_AMBIENT) == TRUE)
    {
        SetSpawnInCondition(NW_FLAG_AMBIENT_ANIMATIONS);
    }
    }

    //DelayCommand(10.0, ApplyEffectToObject(DURATION_TYPE_INSTANT, EffectDeath(), OBJECT_SELF));

    //Post Spawn event requeste
    if (nSpecEvent == 2 || nSpecEvent == 3)
    {
    SignalEvent(OBJECT_SELF,EventUserDefined(EVENT_USER_DEFINED_POSTSPAWN));
    }

}
