//::///////////////////////////////////////////////
//:: Death Script
//:: x4_def_mod_death.nss
//:: Copyright (c) 2014 Bioware Corp.
//:://////////////////////////////////////////////
/*
    This script handles the default behavior
    that occurs when a player dies.

    BK: October 8 2002: Overriden for Expansion
*/
//:://////////////////////////////////////////////
//:: Created By: Brent Knowles
//:: Created On: November 6, 2001
//:: Modified: 69MEH69 SEP2005
//:://////////////////////////////////////////////
#include "69_hench_lib"
#include "_sh_inc_list"
#include "prc_inc_util"

const float DeathPanel_Delay = 6.0;  //Set delay of Death GUI Panel, 6.0 = one round

void ClearAndRun(object oNemesis)
{
    SetLocalInt(oNemesis, "Running", TRUE);
    AssignCommand(oNemesis, ClearAllActions(TRUE));
    AssignCommand(oNemesis, EscapeArea());
}

void DeathPanel(object oPlayer)
{
 if(GetIsDead(oPlayer))
 {
     PopUpDeathGUIPanel(oPlayer,TRUE,TRUE,64521);
 }
}

void Ressurect(effect eEffect, object oPlayer)
{
    ApplyEffectToObject(DURATION_TYPE_INSTANT, eEffect, oPlayer);
    ApplyEffectToObject(DURATION_TYPE_INSTANT, EffectHeal(GetMaxHitPoints()), oPlayer);
    PRCForceRest(oPlayer);
}

void Raise(object oPlayer)
{
        effect eVisual = EffectVisualEffect(VFX_IMP_RESTORATION);

        effect eBad = GetFirstEffect(oPlayer);
        ApplyEffectToObject(DURATION_TYPE_INSTANT,EffectResurrection(),oPlayer);
        ApplyEffectToObject(DURATION_TYPE_INSTANT,EffectHeal(GetMaxHitPoints(oPlayer)), oPlayer);

        //Search for negative effects
        while(GetIsEffectValid(eBad))
        {
            if (GetEffectType(eBad) == EFFECT_TYPE_ABILITY_DECREASE ||
                GetEffectType(eBad) == EFFECT_TYPE_AC_DECREASE ||
                GetEffectType(eBad) == EFFECT_TYPE_ATTACK_DECREASE ||
                GetEffectType(eBad) == EFFECT_TYPE_DAMAGE_DECREASE ||
                GetEffectType(eBad) == EFFECT_TYPE_DAMAGE_IMMUNITY_DECREASE ||
                GetEffectType(eBad) == EFFECT_TYPE_SAVING_THROW_DECREASE ||
                GetEffectType(eBad) == EFFECT_TYPE_SPELL_RESISTANCE_DECREASE ||
                GetEffectType(eBad) == EFFECT_TYPE_SKILL_DECREASE ||
                GetEffectType(eBad) == EFFECT_TYPE_BLINDNESS ||
                GetEffectType(eBad) == EFFECT_TYPE_DEAF ||
                GetEffectType(eBad) == EFFECT_TYPE_PARALYZE ||
                GetEffectType(eBad) == EFFECT_TYPE_NEGATIVELEVEL)
                {
                    //Remove effect if it is negative.
                    RemoveEffect(oPlayer, eBad);
                }
            eBad = GetNextEffect(oPlayer);
        }
        //Fire cast spell at event for the specified target
        SignalEvent(oPlayer, EventSpellCastAt(OBJECT_SELF, SPELL_RESTORATION, FALSE));
        ApplyEffectToObject(DURATION_TYPE_INSTANT, eVisual, oPlayer);
}


void main()
{
    object oPlayer = GetLastPlayerDied();

    //Defeat in a friendly duel with NPC
    if (GetLocalInt(GetArea(oPlayer), "Duel") == TRUE)
    {
        object oObject = GetFirstObjectInArea(GetArea(oPlayer));
        object oNPC;
        while (GetIsObjectValid(oObject))
        {
            if (!GetIsPC(oObject) && GetLocalInt(oObject, "Duel") == TRUE) oNPC = oObject;
            oObject = GetNextObjectInArea(GetArea(oPlayer));
        }
        effect eEffect = SupernaturalEffect(EffectResurrection());
        DelayCommand(3.0, Ressurect(eEffect, oPlayer));
        SetPlotFlag(oNPC, TRUE);
        oObject = GetFirstObjectInArea(GetArea(oPlayer));
        while (GetIsObjectValid(oObject))
        {
            SetImmortal(oObject, FALSE);
            ClearPersonalReputation(oObject, oNPC);
            AssignCommand(oObject, ClearAllActions());
            oObject = GetNextObjectInArea(GetArea(oPlayer));
        }
        SetLocalInt(OBJECT_SELF, "Defeat", FALSE);
        SetLocalInt(OBJECT_SELF, "DuelOver", TRUE);
        DeleteLocalInt(GetArea(OBJECT_SELF), "Duel");
        return;
    }


    int PC_BLEEDING = GetLocalInt(GetModule(), "PC_BLEEDING");
    string sPlayer = GetName(oPlayer);
    float fDelay = DeathPanel_Delay; //Amount of seconds before Death GUI panel pops up
    // * increment global tracking number of times that I died
    SetLocalInt(oPlayer, "NW_L_PLAYER_DIED", GetLocalInt(oPlayer, "NW_L_PLAYER_DIED") + 1);
    if(PC_BLEEDING == 1)
    {
     //Henchman Cleric Code
     int nNum = 1;
     object oHealer = GetHenchman(oPlayer, nNum);
     while(oHealer != OBJECT_INVALID)
     {
       HenchRessurect69(oPlayer, oHealer);
       oHealer = GetHenchman(oPlayer, ++nNum);
     }
    }
    //End Henchman Cleric Code

    // * make friendly to Each of the 3 common factions
    AssignCommand(oPlayer, ClearAllActions());

    if (GetStandardFactionReputation(STANDARD_FACTION_COMMONER, oPlayer) <= 10)
    {   SetLocalInt(oPlayer, "NW_G_Playerhasbeenbad", 10); // * Player bad
        SetStandardFactionReputation(STANDARD_FACTION_COMMONER, 80, oPlayer);
    }
    if (GetStandardFactionReputation(STANDARD_FACTION_MERCHANT, oPlayer) <= 10)
    {   SetLocalInt(oPlayer, "NW_G_Playerhasbeenbad", 10); // * Player bad
        SetStandardFactionReputation(STANDARD_FACTION_MERCHANT, 80, oPlayer);
    }
    if (GetStandardFactionReputation(STANDARD_FACTION_DEFENDER, oPlayer) <= 10)
    {   SetLocalInt(oPlayer, "NW_G_Playerhasbeenbad", 10); // * Player bad
        SetStandardFactionReputation(STANDARD_FACTION_DEFENDER, 80, oPlayer);
    }

    //Make nemesis NPCs go away when they kill the PC
    object oArea = GetArea(oPlayer);
    if (GetLocalInt(oArea, "NO_ESCAPE") == TRUE)
    {
        object oNPC = GetFirstObjectInArea(oArea);
        object oNemesis;
        string sString;
        while (GetIsObjectValid(oNPC))
        {
            if (GetLocalString(oNPC, "sOwner") == CharacterDB(oPlayer))
            {
                oNemesis = oNPC;
                break;
            }
            oNPC = GetNextObjectInArea(oArea);
        }
        if (GetIsObjectValid(oNemesis))
        {
            SetPlotFlag(oNemesis, TRUE);
            DeleteLocalInt(oArea, "NO_ESCAPE");
            switch (Random(3))
                {
                    case 0: sString = "You got what you deserved, scum!"; break;
                    case 1: sString = "Taste my wrath."; break;
                    case 2: sString = "Pathetic fool."; break;
                }
            AssignCommand(oNemesis, SpeakString(sString));
            DelayCommand(5.0, ClearAndRun(oNemesis));
        }
    }

    //Get the last killer of the PC and check if it's another PC or their associates
    int nPVPDeath = FALSE;
    object oKiller = GetLastHostileActor(oPlayer);
    while (GetIsObjectValid(GetMaster(oKiller))) //Loop through until we find one that doesn't have a master
    {
        oKiller=GetMaster(oKiller);
    }
    if (GetIsPC(oKiller)) nPVPDeath = TRUE;
    string sKillerName;
    int nKillerLvl;
    if (nPVPDeath == TRUE)
    {
        sKillerName = GetName(oKiller);
        nKillerLvl = GetHitDice(oKiller);
    }

    //Apply respawn penalties
    string sDatabase = GetLocalString(GetModule(), "DB");
    string sDCommu = GetCampaignString(sDatabase, "difficulty_death"); //commoner, adventurer, hero, legend, demigod
    int nGold;
    object oItem;
    object oLootBag;
    if (sDCommu == "COMMONER")
    {
        //create a loot bag if applicable
        if (nPVPDeath == TRUE) oLootBag = CreateObject(OBJECT_TYPE_PLACEABLE, "anc_pcloot", GetLocation(oPlayer));
    }
    if (sDCommu == "ADVENTURER")
    {
        //create a loot bag if applicable
        if (nPVPDeath == TRUE) oLootBag = CreateObject(OBJECT_TYPE_PLACEABLE, "anc_pcloot", GetLocation(oPlayer));

        //drop gold
        nGold = GetGold(oPlayer)/2;
        if (nGold > 0)
        {
            if (nPVPDeath == TRUE) oItem = CreateItemOnObject("anc_it_goldpouch", oLootBag);
            else oItem = CreateObject(OBJECT_TYPE_ITEM, "anc_it_goldpouch", GetLocation(oPlayer));
            SetLocalInt(oItem, "Gold", nGold);
        }
        AssignCommand(oPlayer, TakeGoldFromCreature(nGold, oPlayer, TRUE));
    }
    if (sDCommu == "HERO")
    {
        //create a loot bag if applicable
        if (nPVPDeath == TRUE) oLootBag = CreateObject(OBJECT_TYPE_PLACEABLE, "anc_pcloot", GetLocation(oPlayer));

        //drop gold
        nGold = GetGold(oPlayer);
        if (nGold > 0)
        {
            if (nPVPDeath == TRUE) oItem = CreateItemOnObject("anc_it_goldpouch", oLootBag);
            else oItem = CreateObject(OBJECT_TYPE_ITEM, "anc_it_goldpouch", GetLocation(oPlayer));
            SetLocalInt(oItem, "Gold", nGold);
        }
        AssignCommand(oPlayer, TakeGoldFromCreature(nGold, oPlayer, TRUE));
    }
    if (sDCommu == "LEGEND")
    {
        //create a loot bag
        oLootBag = CreateObject(OBJECT_TYPE_PLACEABLE, "anc_pcloot", GetLocation(oPlayer));

        //drop gold
        nGold = GetGold(oPlayer);
        if (nGold > 0)
        {
            oItem = CreateItemOnObject("anc_it_goldpouch", oLootBag);
            SetLocalInt(oItem, "Gold", nGold);
        }
        AssignCommand(oPlayer, TakeGoldFromCreature(nGold, oPlayer, TRUE));

        //drop a randomly selected eq item
        object oList = ListCreate("DroppableItems");
        int i;
        for (i = 1; i <= 11; i++)
        {
            switch (i)
            {
                case 1: oItem = GetItemInSlot(INVENTORY_SLOT_CHEST, oPlayer); break;
                case 2: oItem = GetItemInSlot(INVENTORY_SLOT_HEAD, oPlayer); break;
                case 3: oItem = GetItemInSlot(INVENTORY_SLOT_RIGHTHAND, oPlayer); break;
                case 4: oItem = GetItemInSlot(INVENTORY_SLOT_LEFTHAND, oPlayer); break;
                case 5: oItem = GetItemInSlot(INVENTORY_SLOT_BELT, oPlayer); break;
                case 6: oItem = GetItemInSlot(INVENTORY_SLOT_ARMS, oPlayer); break;
                case 7: oItem = GetItemInSlot(INVENTORY_SLOT_LEFTRING, oPlayer); break;
                case 8: oItem = GetItemInSlot(INVENTORY_SLOT_RIGHTRING, oPlayer); break;
                case 9: oItem = GetItemInSlot(INVENTORY_SLOT_NECK, oPlayer); break;
                case 10: oItem = GetItemInSlot(INVENTORY_SLOT_CLOAK, oPlayer); break;
                case 11: oItem = GetItemInSlot(INVENTORY_SLOT_BOOTS, oPlayer); break;
            }
            if (GetIsObjectValid(oItem)) ListAddObject(oList, oItem);
        }
        int nItems = ListGetElementCount(oList);
        if (nItems > 0)
        {
            oItem = ListGetObject(oList, Random(nItems)+1);
            CopyItem(oItem, oLootBag, TRUE);
            DestroyObject(oItem);
        }
        DestroyObject(oList);
    }
    if (sDCommu == "DEMIGOD")
    {
        //create a loot bag
        oLootBag = CreateObject(OBJECT_TYPE_PLACEABLE, "anc_pcloot", GetLocation(oPlayer));

        //drop gold
        nGold = GetGold(oPlayer);
        if (nGold > 0)
        {
            oItem = CreateItemOnObject("anc_it_goldpouch", oLootBag);
            SetLocalInt(oItem, "Gold", nGold);
        }
        AssignCommand(oPlayer, TakeGoldFromCreature(nGold, oPlayer, TRUE));

        //drop all items
        int i;
        for (i = 1; i < 15; i++)
            {
                switch (i)
                {
                case 1: oItem = GetItemInSlot(INVENTORY_SLOT_ARMS, oPlayer); break;
                case 2: oItem = GetItemInSlot(INVENTORY_SLOT_ARROWS, oPlayer); break;
                case 3: oItem = GetItemInSlot(INVENTORY_SLOT_BELT, oPlayer); break;
                case 4: oItem = GetItemInSlot(INVENTORY_SLOT_BOLTS, oPlayer); break;
                case 5: oItem = GetItemInSlot(INVENTORY_SLOT_BOOTS, oPlayer); break;
                case 6: oItem = GetItemInSlot(INVENTORY_SLOT_BULLETS, oPlayer); break;
                case 7: oItem = GetItemInSlot(INVENTORY_SLOT_CHEST, oPlayer); break;
                case 8: oItem = GetItemInSlot(INVENTORY_SLOT_CLOAK, oPlayer); break;
                case 9: oItem = GetItemInSlot(INVENTORY_SLOT_HEAD, oPlayer); break;
                case 10: oItem = GetItemInSlot(INVENTORY_SLOT_LEFTHAND, oPlayer); break;
                case 11: oItem = GetItemInSlot(INVENTORY_SLOT_LEFTRING, oPlayer); break;
                case 12: oItem = GetItemInSlot(INVENTORY_SLOT_NECK, oPlayer); break;
                case 13: oItem = GetItemInSlot(INVENTORY_SLOT_RIGHTHAND, oPlayer); break;
                case 14: oItem = GetItemInSlot(INVENTORY_SLOT_RIGHTRING, oPlayer); break;
                }
                if (GetIsObjectValid(oItem))
                {
                    CopyItem(oItem, oLootBag, TRUE);
                    DestroyObject(oItem);
                }
            }
        oItem = GetFirstItemInInventory(oPlayer);
        while (GetIsObjectValid(oItem))
        {
            CopyItem(oItem, oLootBag, TRUE);
            DestroyObject(oItem);
            oItem = GetNextItemInInventory(oPlayer);
        }
    }

    //Create a PVP trophy in the slain PC's inventory
    if (nPVPDeath == TRUE)
    {
        oItem = CreateItemOnObject("anc_pvptrophy", oLootBag);
        string sName = "Tooth of " + GetName(oPlayer);
        string sDescription = "This is a tooth of " + GetName(oPlayer) + " (" + IntToString(GetHitDice(oPlayer)) + ")"
                              + ", slain by " + sKillerName + " (" + IntToString(nKillerLvl) + ").";
        SetName(oItem, sName);
        SetDescription(oItem, sDescription);
    }

    // * BK: Automation Control. Autopcs ignore death
    if(GetLocalInt(oPlayer, "NW_L_AUTOMATION") == 10)
    {
        Raise(oPlayer);
        DelayCommand(1.0, ExecuteScript("crawl", OBJECT_SELF));
        return; // Raise and return
    }
    if(GetHenchmanTotal69(oPlayer) > 0)
    {
     fDelay = 30.0 + fDelay;
    }
    DelayCommand(fDelay, DeathPanel(oPlayer));

}
