//::///////////////////////////////////////////////
//:: OnLoad Script
//:: x4_def_mod_load
//:: (c) 2003 Bioware Corp.
//:://////////////////////////////////////////////
/*
    Put into: OnModuleLoad Event

    This example script demonstrates how to tweak the
    behavior of several subsystems in your module.

    For more information, please check x2_inc_switches
    which holds definitions for several variables that
    can be set on modules, creatures, doors or waypoints
    to change the default behavior of Bioware scripts.

    Warning:
    Using some of these switches may change your games
    balancing and may introduce bugs or instabilities. We
    recommend that you only use these switches if you
    know what you are doing. Consider these features
    unsupported!

    Please do NOT report any bugs you experience while
    these switches have been changed from their default
    positions.

    Make sure you visit the forums at nwn.bioware.com
    to find out more about these scripts.

*/
//:://////////////////////////////////////////////
//:: Created By: Georg Zoeller
//:: Created On: 2003-07-16
//:: Modified: 69MEH69
//:://////////////////////////////////////////////

#include "nwnx_files"
#include "_sh_inc_list"
#include "x2_inc_switches"
#include "x2_inc_restsys"
#include "global_inc"

void SaveDate()
{
    int nDate = CurrentDateTime();
    string sDB = GetLocalString(GetModule(), "DB");
    SetCampaignInt(sDB, "DATE", nDate);
    float fDelay = 0.5 * HoursToSeconds(1);
    DelayCommand(fDelay, SaveDate());
}

void AddPokerGame()
{
    object oPC = GetFirstPC();
    int nGames;
    string sDB;
    while (GetIsObjectValid(oPC))
    {
        sDB = CharacterDB(oPC);
        nGames = GetCampaignInt(sDB, "POKER_GAMES");
        if (nGames < 10) SetCampaignInt(sDB, "POKER_GAMES", nGames+1);
        oPC = GetNextPC();
    }
    float fDelay = HoursToSeconds(3);
    DelayCommand(fDelay, AddPokerGame());
}

void GenerateQuests()
{
    object oPC = GetFirstPC();
    while (GetIsObjectValid(oPC))
    {
        if (Random(10) == 0) MakeQuestAvailable(oPC); //10% every half an hour (1 minute) of generating a quest
        oPC = GetNextPC();
    }
    float fDelay = 0.5 * HoursToSeconds(1);
    DelayCommand(fDelay, GenerateQuests());
}

void ReturnHouseToAgency(object oModule, string sDatabase, string sHouseName)
{
    DeleteCampaignVariable(sDatabase, "H_"+sHouseName+"_OWNER");
    DeleteCampaignVariable(sDatabase, "H_"+sHouseName+"_AUTH_1");
    DeleteCampaignVariable(sDatabase, "H_"+sHouseName+"_AUTH_2");
    DeleteCampaignVariable(sDatabase, "H_"+sHouseName+"_AUTH_3");
    DeleteCampaignVariable(sDatabase, "H_"+sHouseName+"_AUTH_4");
    DeleteCampaignVariable(sDatabase, "H_"+sHouseName+"_AUTH_5");
    DeleteCampaignVariable(sDatabase, "H_"+sHouseName+"_AUTH_6");
    DeleteCampaignVariable(sDatabase, "H_"+sHouseName+"_AUTH_7");
    DeleteCampaignVariable(sDatabase, "H_"+sHouseName+"_AUTH_8");
    DeleteCampaignVariable(sDatabase, "H_"+sHouseName+"_AUTH_9");
    DeleteCampaignVariable(sDatabase, "H_"+sHouseName+"_AUTH_10");
    DeleteCampaignVariable(sDatabase, "TIME_"+sHouseName);
}


void UpdateHouses()
{
    object oMod = GetModule();
    string sDat = GetLocalString(oMod, "DB");
    int nDays = GetLocalInt(oMod, "HOUSE_DAYS")*86400;
    int nSystemTime = GetSystemTime();
    int nStoredTime;
    int nDifference;
    if(GetLocalInt(oMod, "UpdateHouses") == TRUE)
    {
    if(GetLocalInt(oMod, "TIME_ARKENSTONE") != 0) SetCampaignInt(sDat, "TIME_ARKENSTONE", GetLocalInt(oMod, "TIME_ARKENSTONE"));
    if(GetLocalInt(oMod, "TIME_AUBERAGE") != 0) SetCampaignInt(sDat, "TIME_AUBERAGE", GetLocalInt(oMod, "TIME_AUBERAGE"));
    if(GetLocalInt(oMod, "TIME_BALDAM") != 0) SetCampaignInt(sDat, "TIME_BALDAM", GetLocalInt(oMod, "TIME_BALDAM"));
    if(GetLocalInt(oMod, "TIME_DRAGONWOLD") != 0) SetCampaignInt(sDat, "TIME_DRAGONWOLD", GetLocalInt(oMod, "TIME_DRAGONWOLD"));
    if(GetLocalInt(oMod, "TIME_WESTRON") != 0) SetCampaignInt(sDat, "TIME_WESTRON", GetLocalInt(oMod, "TIME_WESTRON"));
    if(GetLocalInt(oMod, "TIME_HEIMDOL") != 0) SetCampaignInt(sDat, "TIME_HEIMDOL", GetLocalInt(oMod, "TIME_HEIMDOL"));
    if(GetLocalInt(oMod, "TIME_FOHENWIND") != 0) SetCampaignInt(sDat, "TIME_FOHENWIND", GetLocalInt(oMod, "TIME_FOHENWIND"));
    if(GetLocalInt(oMod, "TIME_SILBUAM") != 0) SetCampaignInt(sDat, "TIME_SILBUAM", GetLocalInt(oMod, "TIME_SILBUAM"));
    DeleteLocalInt(oMod, "UpdateHouses");
    DeleteLocalInt(oMod, "TIME_ARKENSTONE");
    DeleteLocalInt(oMod, "TIME_AUBERAGE");
    DeleteLocalInt(oMod, "TIME_BALDAM");
    DeleteLocalInt(oMod, "TIME_DRAGONWOLD");
    DeleteLocalInt(oMod, "TIME_WESTRON");
    DeleteLocalInt(oMod, "TIME_HEIMDOL");
    DeleteLocalInt(oMod, "TIME_FOHENWIND");
    DeleteLocalInt(oMod, "TIME_SILBUAM");
    }
    nStoredTime = GetCampaignInt(sDat, "TIME_ARKENSTONE");
    nDifference = nSystemTime - nStoredTime;
    if(nDifference > nDays) ReturnHouseToAgency(oMod, sDat, "ARKENSTONE");
    nStoredTime = GetCampaignInt(sDat, "TIME_AUBERAGE");
    nDifference = nSystemTime - nStoredTime;
    if(nDifference > nDays) ReturnHouseToAgency(oMod, sDat, "AUBERAGE");
    nStoredTime = GetCampaignInt(sDat, "TIME_BALDAM");
    nDifference = nSystemTime - nStoredTime;
    if(nDifference > nDays) ReturnHouseToAgency(oMod, sDat, "BALDAM");
    nStoredTime = GetCampaignInt(sDat, "TIME_DRAGONWOLD");
    nDifference = nSystemTime - nStoredTime;
    if(nDifference > nDays) ReturnHouseToAgency(oMod, sDat, "DRAGONWOLD");
    nStoredTime = GetCampaignInt(sDat, "TIME_WESTRON");
    nDifference = nSystemTime - nStoredTime;
    if(nDifference > nDays) ReturnHouseToAgency(oMod, sDat, "WESTRON");
    nStoredTime = GetCampaignInt(sDat, "TIME_HEIMDOL");
    nDifference = nSystemTime - nStoredTime;
    if(nDifference > nDays) ReturnHouseToAgency(oMod, sDat, "HEIMDOL");
    nStoredTime = GetCampaignInt(sDat, "TIME_FOHENWIND");
    nDifference = nSystemTime - nStoredTime;
    if(nDifference > nDays) ReturnHouseToAgency(oMod, sDat, "FOHENWIND");
    nStoredTime = GetCampaignInt(sDat, "TIME_SILBUAM");
    nDifference = nSystemTime - nStoredTime;
    if(nDifference > nDays) ReturnHouseToAgency(oMod, sDat, "SILBUAM");
    DelayCommand(6.0, UpdateHouses());
}

void main()
{
   object oModule = GetModule();
   if (GetLocalInt(oModule, "STARTED") == TRUE) SetLocalInt(oModule, "SAVELOADED", TRUE);
   SetLocalInt(oModule, "STARTED", TRUE); //this is used in OnClientEnter to send PCs a warning on save load
   string sDatabase = GetLocalString(oModule, "DB");
   //Henchman Kit Definitions 69MEH69

   //Set the module's spellhook
   SetLocalString(oModule, "X2_S_UD_SPELLSCRIPT", "x4_spellhook");

   //Sets Database Name for your Module
   //Useful for transfering henchmen from module to module
   //Insure that all the transfering modules use the same database
   string sDB = sDatabase;

   SetLocalString(GetModule(), "X0_CAMPAIGN_DB", sDB);

   //Sets testing mode
   //Useful for debugging and/or viewing feedback
   //Set TEST_MODE = 1 to initiate test mode
   //Set TEST_MODE = 0 to disable test mode (Default)
   int TEST_MODE = 0;

   SetLocalInt(GetModule(), "TEST_MODE", TEST_MODE);

   //PC death option:
   //Set PC_BLEEDING = 1 to initiate PC bleeding (-10 HP) & Raise Dead/Resurrection by a healer henchman
   //Set PC_BLEEDING = 0 to initiate default BIOWARE PC death
   int PC_BLEEDING = 0;

   SetLocalInt(GetModule(), "PC_BLEEDING", PC_BLEEDING);

   //Henchman Death Option:
   //Set HENCH_BLEED = 3 for henchman to respawn at PC master's location after 30.0 seconds
   //Set HENCH_BLEED = 2 for henchman destruction, henchman will be destroyed upon death
   //Set HENCH_BLEED = 1 for henchman bleeding
   //Set HENCH_BLEED = 0 for respawn death, henchman will respawn
   //    to location defined in spawn script
   int HENCH_BLEED = 0;

   SetLocalInt(GetModule(), "HENCH_BLEED", HENCH_BLEED);

   //Set HENCH_SALVATION = 1 to allow for the possibility of
   //henchman recovering from wounds each round
   //Set HENCH_SALVATION = 0 to allow only one check for the
   //possibility of henchman recovering from wounds
   //Only available if HENCH_BLEED = 1;
   int HENCH_SALVATION  = 0;

   SetLocalInt(GetModule(), "HENCH_SALVATION", HENCH_SALVATION);

   //Sets whether henchman will gain own XP or level up with PC
   //Will initiate the 3rd edition DMG XP code
   //Adjust the XP slider to 0 in Module Properties.
   //0 = Levels up when PC levels up
   //1 = Henchman gains own personal XP
   int nHenchXP = 0;

   SetLocalInt(GetModule(), "HENCHXP", nHenchXP);

   //Sets whether henchman retain level or level down to level 1
   //Will destroy NPC and create original level 1 version at original spawn location
   //determined by waypoint with TAG: "WP_Home_'NPC TAG'"
   //OR Henchman will be destroyed permanently
   //The 69_hench_quit script will initiate on the following dialog node:
   // "No, I work better alone."
   //0 = Retains leveled up henchman
   //1 = NPC is destroyed and level 1 version is created at waypoint
   //2 = NPC is destroyed
   int nHenchQuit = 1;

   SetLocalInt(GetModule(), "HENCHQUIT", nHenchQuit);

   //Sets whether henchmen will autolevel up at hire:
   //0 = must level up henchman through dialog
   //1 = henchman will automatically level up at hire
   int nAutoLevelup = 1;

   SetLocalInt(GetModule(), "nAutoLevelup", nAutoLevelup);

   //Sets Leadership requirement upon module:
   //0 = No Leadership, 1 = Yes Leadership
   int nLeadership = 0;

   SetLocalInt(GetModule(), "nLeadership", nLeadership);

   if(nLeadership == 1)
   {
     SetMaxHenchmen(100);  //Do Not Edit
   }

   //Sets Loyalty requirement upon module:
   //0 = No Loyalty, 1 = Yes Loyalty
   int nLoyalty = 0;

   SetLocalInt(GetModule(), "nLoyalty", nLoyalty);

   //Maximum Henchmen
   //Sets the number of henchman a PC may have:
   //(Leadership requirement must be set to 0)
   //0,1,2...
   int nNumHench = 2;

   if(nLeadership == 0)
   {
    SetMaxHenchmen(nNumHench);
   }

   //End Henchman Kit Definitions 69MEH69

   //Other Definitions
   if (GetGameDifficulty() ==  GAME_DIFFICULTY_CORE_RULES || GetGameDifficulty() ==  GAME_DIFFICULTY_DIFFICULT)
   {
        // * Setting the switch below will enable a seperate Use Magic Device Skillcheck for
        // * rogues when playing on Hardcore+ difficulty. This only applies to scrolls
        SetModuleSwitch (MODULE_SWITCH_ENABLE_UMD_SCROLLS, TRUE);

       // * Activating the switch below will make AOE spells hurt neutral NPCS by default
       // SetModuleSwitch (MODULE_SWITCH_AOE_HURT_NEUTRAL_NPCS, TRUE);
   }

   // * AI: Activating the switch below will make the creaures using the WalkWaypoint function
   // * able to walk across areas
   // SetModuleSwitch (MODULE_SWITCH_ENABLE_CROSSAREA_WALKWAYPOINTS, TRUE);

   // * Spells: Activating the switch below will make the Glyph of Warding spell behave differently:
   // * The visual glyph will disappear after 6 seconds, making them impossible to spot
   // SetModuleSwitch (MODULE_SWITCH_ENABLE_INVISIBLE_GLYPH_OF_WARDING, TRUE);

   // * Craft Feats: Want 50 charges on a newly created wand? We found this unbalancing,
   // * but since it is described this way in the book, here is the switch to get it back...
   // SetModuleSwitch (MODULE_SWITCH_ENABLE_CRAFT_WAND_50_CHARGES, TRUE);

   // * Craft Feats: Use this to disable Item Creation Feats if you do not want
   // * them in your module
   // SetModuleSwitch (MODULE_SWITCH_DISABLE_ITEM_CREATION_FEATS, TRUE);

   // * Palemaster: Deathless master touch in PnP only affects creatures up to a certain size.
   // * We do not support this check for balancing reasons, but you can still activate it...
   // SetModuleSwitch (MODULE_SWITCH_SPELL_CORERULES_DMASTERTOUCH, TRUE);

   // * Epic Spellcasting: Some Epic spells feed on the liveforce of the caster. However this
   // * did not fit into NWNs spell system and was confusing, so we took it out...
   // SetModuleSwitch (MODULE_SWITCH_EPIC_SPELLS_HURT_CASTER, TRUE);

   // * Epic Spellcasting: Some Epic spells feed on the liveforce of the caster. However this
   // * did not fit into NWNs spell system and was confusing, so we took it out...
   // SetModuleSwitch (MODULE_SWITCH_RESTRICT_USE_POISON_TO_FEAT, TRUE);

    // * Spellcasting: Some people don't like caster's abusing expertise to raise their AC
    // * Uncommenting this line will drop expertise mode whenever a spell is cast by a player
    // SetModuleSwitch (MODULE_VAR_AI_STOP_EXPERTISE_ABUSE, TRUE);


    // * Item Event Scripts: The game's default event scripts allow routing of all item related events
    // * into a single file, based on the tag of that item. If an item's tag is "test", it will fire a
    // * script called "test" when an item based event (equip, unequip, acquire, unacquire, activate,...)
    // * is triggered. Check "x2_it_example.nss" for an example.
    // * This feature is disabled by default.
   SetModuleSwitch (MODULE_SWITCH_ENABLE_TAGBASED_SCRIPTS, TRUE);

   if (GetModuleSwitchValue (MODULE_SWITCH_ENABLE_TAGBASED_SCRIPTS) == TRUE)
   {
        // * If Tagbased scripts are enabled, and you are running a Local Vault Server
        // * you should use the line below to add a layer of security to your server, preventing
        // * people to execute script you don't want them to. If you use the feature below,
        // * all called item scrips will be the prefix + the Tag of the item you want to execute, up to a
        // * maximum of 16 chars, instead of the pure tag of the object.
        // * i.e. without the line below a user activating an item with the tag "test",
        // * will result in the execution of a script called "test". If you uncomment the line below
        // * the script called will be "1_test.nss"
        // SetUserDefinedItemEventPrefix("1_");

   }

   // * This initializes Bioware's wandering monster system as used in Hordes of the Underdark
   // * You can deactivate it, making your module load faster if you do not use it.
   // * If you want to use it, make sure you set "x2_mod_def_rest" as your module's OnRest Script
   // SetModuleSwitch (MODULE_SWITCH_USE_XP2_RESTSYSTEM, TRUE);

   if (GetModuleSwitchValue(MODULE_SWITCH_USE_XP2_RESTSYSTEM) == TRUE)
   {

       // * This allows you to specify a different 2da for the wandering monster system.
       // SetWanderingMonster2DAFile("des_restsystem");

       //* Do not change this line.
       WMBuild2DACache();
   }

//These lines ensure horses work as intended.
SetLocalInt(GetModule(), "X3_HORSE_ENABLE_ACBOOST", TRUE);
SetLocalInt(GetModule(), "X3_HORSE_ENABLE_HPBOOST", TRUE);
SetLocalInt(GetModule(), "X3_MOUNTS_EXTERNAL_ONLY", TRUE);

//Set a variable to prevent the random area OnEnter event from firing after loading a saved game
SetLocalInt(oModule, "LoadCooldown", TRUE);
DelayCommand(3.0, DeleteLocalInt(oModule, "LoadCooldown"));

//Create lists to store non-occupied areas
if (GetLocalInt(OBJECT_SELF, "ListsCreated") == FALSE)
{
    SetLocalInt(OBJECT_SELF, "ListsCreated", TRUE);
    //Normal areas for Green Coast
    object oList1 = ListCreate("gcoast_free");
    int i;
    for (i = 1; i <= 21; i++)
    {
        ListAddString(oList1, "1_"+IntToString(i));
    }
    SetLocalObject(OBJECT_SELF, "sList1", oList1);
    //Quest areas for Green Coast
    object oList1q = ListCreate("gcoast_quest");
    for (i = 25; i <= 44; i++)
    {
        ListAddString(oList1q, "1_"+IntToString(i));
    }
    SetLocalObject(OBJECT_SELF, "sList1q", oList1q);
    //Dungeon entrance areas for Green Coast
    object oList1d = ListCreate("gcoast_dung");
    for (i = 45; i <= 64; i++)
    {
        ListAddString(oList1d, "1_"+IntToString(i));
    }
    SetLocalObject(OBJECT_SELF, "sList1d", oList1d);

    //Dungeon areas
    object oListDung = ListCreate("dungeons");
    for (i = 1; i <= 60; i++)
    {
        ListAddString(oListDung, "d_"+IntToString(i));
    }
    SetLocalObject(OBJECT_SELF, "sListDung", oListDung);
}


// Retrieve world difficulty settings from the database
if (GetCampaignInt(sDatabase, "db_init") == 1)
{
string sDDeath = GetCampaignString(sDatabase, "difficulty_death");
string sDExplo = GetCampaignString(sDatabase, "difficulty_explo");
string sDEnemy = GetCampaignString(sDatabase, "difficulty_enemy");
string sDAdven = GetCampaignString(sDatabase, "difficulty_adven");
string sDCommu = GetCampaignString(sDatabase, "difficulty_commu");
SetLocalString(oModule, "mDiffDeath", sDDeath);
SetLocalString(oModule, "mDiffExplo", sDExplo);
SetLocalString(oModule, "mDiffEnemy", sDEnemy);
SetLocalString(oModule, "mDiffAdven", sDAdven);
SetLocalString(oModule, "mDiffCommu", sDCommu);
}

// Make sure the mode of operation is either LOCAL or ONLINE and not anything else
if (GetLocalString(oModule, "MODE") != "ONLINE") SetLocalString(oModule, "MODE", "LOCAL");

//Retrieve game date from the database
string sDat = GetLocalString(GetModule(), "DB");
if (GetCampaignInt(sDat, "DATE") != 0)
{
    int nDate = GetCampaignInt(sDat, "DATE");
    SetDateTime(nDate);
}
else
{
    SetCalendar(1260, 4, 26);
    SetTime(6, 0, 0, 0);
}
//Start writing down game date in the database every game hour
float fDelay = 0.5 * HoursToSeconds(1);
DelayCommand(fDelay, SaveDate());

//Spawn the festival merchant if appropriate
if (GetCampaignString(sDat, "1_EVENT") == "festival")
{
    string sTown = GetCampaignString(sDat, "1_EV_DAT1");
    if (GetCampaignInt(sDat, sTown+"_FESTIVAL") == TRUE)
    {
        object oNPC = CreateObject(OBJECT_TYPE_CREATURE, "anc_ultimatemerc", GetLocation(GetWaypointByTag(sTown+"_ultmerch")));
        SetLocalString(oNPC, "Town", sTown);
    }
}
if (GetCampaignString(sDat, "2_EVENT") == "festival")
{
    string sTown = GetCampaignString(sDat, "2_EV_DAT1");
    if (GetCampaignInt(sDat, sTown+"_FESTIVAL") == TRUE)
    {
        object oNPC = CreateObject(OBJECT_TYPE_CREATURE, "anc_ultimatemerc", GetLocation(GetWaypointByTag(sTown+"_ultmerch")));
        SetLocalString(oNPC, "Town", sTown);
    }
}
if (GetCampaignString(sDat, "3_EVENT") == "festival")
{
    string sTown = GetCampaignString(sDat, "3_EV_DAT1");
    if (GetCampaignInt(sDat, sTown+"_FESTIVAL") == TRUE)
    {
        object oNPC = CreateObject(OBJECT_TYPE_CREATURE, "anc_ultimatemerc", GetLocation(GetWaypointByTag(sTown+"_ultmerch")));
        SetLocalString(oNPC, "Town", sTown);
    }
}


//Set each town's economy to 5 if it's 0
if (GetCampaignInt(sDat, "ALVERTON_ECONOMY") == 0) SetCampaignInt(sDat, "ALVERTON_ECONOMY", 5); //TEST - add more

//Start generating quests (and poker games!) for PCs
GenerateQuests();
AddPokerGame();

//Start generating global events
DelayCommand(600.0, GlobalEvents(600.0));

//This part only runs if the module is set to ONLINE mode
//It's a pseudoheartbeat that runs once per hour and returns estates to the agency
//if no one visited them in a number of real-time days defined in a module's string "HOUSE_DAYS"
    if (GetLocalString(oModule, "MODE") == "ONLINE")
    {
    int nExtraDays = 86400*GetLocalInt(oModule, "EXTRA_DAYS");
    if(nExtraDays != 0 && GetCampaignInt(sDatabase, "TIME_AUBERAGE") != 0) SetCampaignInt(sDatabase, "TIME_AUBERAGE", GetCampaignInt(sDatabase, "TIME_AUBERAGE")+nExtraDays);
    if(nExtraDays != 0 && GetCampaignInt(sDatabase, "TIME_SILBUAM") != 0) SetCampaignInt(sDatabase, "TIME_SILBUAM", GetCampaignInt(sDatabase, "TIME_SILBUAM")+nExtraDays);
    if(nExtraDays != 0 && GetCampaignInt(sDatabase, "TIME_FOHENWIND") != 0) SetCampaignInt(sDatabase, "TIME_FOHENWIND", GetCampaignInt(sDatabase, "TIME_FOHENWIND")+nExtraDays);
    if(nExtraDays != 0 && GetCampaignInt(sDatabase, "TIME_WESTRON") != 0) SetCampaignInt(sDatabase, "TIME_WESTRON", GetCampaignInt(sDatabase, "TIME_WESTRON")+nExtraDays);
    if(nExtraDays != 0 && GetCampaignInt(sDatabase, "TIME_HEIMDOL") != 0) SetCampaignInt(sDatabase, "TIME_HEIMDOL", GetCampaignInt(sDatabase, "TIME_HEIMDOL")+nExtraDays);
    if(nExtraDays != 0 && GetCampaignInt(sDatabase, "TIME_BALDAM") != 0) SetCampaignInt(sDatabase, "TIME_BALDAM", GetCampaignInt(sDatabase, "TIME_BALDAM")+nExtraDays);
    if(nExtraDays != 0 && GetCampaignInt(sDatabase, "TIME_ARKENSTONE") != 0) SetCampaignInt(sDatabase, "TIME_ARKENSTONE", GetCampaignInt(sDatabase, "TIME_ARKENSTONE")+nExtraDays);
    if(nExtraDays != 0 && GetCampaignInt(sDatabase, "TIME_DRAGONWOLD") != 0) SetCampaignInt(sDatabase, "TIME_DRAGONWOLD", GetCampaignInt(sDatabase, "TIME_DRAGONWOLD")+nExtraDays);
    UpdateHouses();
    }

}
