//::///////////////////////////////////////////////
//:: Name x2_def_spawn
//:: Copyright (c) 2001 Bioware Corp.
//:://////////////////////////////////////////////
/*
    Default On Spawn script


    2003-07-28: Georg Zoeller:

    If you set a ninteger on the creature named
    "X2_USERDEFINED_ONSPAWN_EVENTS"
    The creature will fire a pre and a post-spawn
    event on itself, depending on the value of that
    variable
    1 - Fire Userdefined Event 1510 (pre spawn)
    2 - Fire Userdefined Event 1511 (post spawn)
    3 - Fire both events

    2007-12-31: Deva Winblood
    Modified to look for X3_HORSE_OWNER_TAG and if
    it is defined look for an NPC with that tag
    nearby or in the module (checks near first).
    It will make that NPC this horse's master.

*/
//:://////////////////////////////////////////////
//:: Created By: Keith Warner, Georg Zoeller
//:: Created On: June 11/03
//:://////////////////////////////////////////////

const int EVENT_USER_DEFINED_PRESPAWN = 1510;
const int EVENT_USER_DEFINED_POSTSPAWN = 1511;


#include "x2_inc_switches"
#include "x0_i0_henchman"

void main()
{
    string sTag;
    object oNPC;
    // User defined OnSpawn event requested?
    int nSpecEvent = GetLocalInt(OBJECT_SELF,"X2_USERDEFINED_ONSPAWN_EVENTS");


    // Pre Spawn Event requested
    if (nSpecEvent == 1  || nSpecEvent == 3  )
    {
    SignalEvent(OBJECT_SELF,EventUserDefined(EVENT_USER_DEFINED_PRESPAWN ));
    }

    sTag=GetLocalString(OBJECT_SELF,"X3_HORSE_OWNER_TAG");
    if (GetStringLength(sTag)>0)
    { // look for master
        oNPC=GetNearestObjectByTag(sTag);
        if (GetIsObjectValid(oNPC)&&GetObjectType(oNPC)==OBJECT_TYPE_CREATURE)
        { // master found
            AddHenchman(oNPC);
        } // master found
        else
        { // look in module
            oNPC=GetObjectByTag(sTag);
            if (GetIsObjectValid(oNPC)&&GetObjectType(oNPC)==OBJECT_TYPE_CREATURE)
            { // master found
                AddHenchman(oNPC);
            } // master found
            else
            { // master does not exist - remove X3_HORSE_OWNER_TAG
                DeleteLocalString(OBJECT_SELF,"X3_HORSE_OWNER_TAG");
            } // master does not exist - remove X3_HORSE_OWNER_TAG
        } // look in module
    } // look for master

    /*  Fix for the new golems to reduce their number of attacks */

    int nNumber = GetLocalInt(OBJECT_SELF,CREATURE_VAR_NUMBER_OF_ATTACKS);
    if (nNumber >0 )
    {
        SetBaseAttackBonus(nNumber);
    }

    //We want these demons to scale in level
    int nLevel = GetLocalInt(GetArea(OBJECT_SELF), "Level");
    if (nLevel > 3) nLevel = nLevel - 3;
    LevelHenchmanUpTo(OBJECT_SELF, nLevel, CLASS_TYPE_INVALID, 0, GetCreatureStartingPackage(OBJECT_SELF), PACKAGE_INVALID);

    //Buff them, we don't want them to be too weak
    effect eWarriorBuff;
    itemproperty iKeen;
    itemproperty iMassive;
    eWarriorBuff = EffectAttackIncrease(1);
    if (GetHitDice(oNPC) >= 10) eWarriorBuff = EffectAttackIncrease(4);
    if (GetHitDice(oNPC) >= 4) eWarriorBuff = EffectLinkEffects(EffectDamageIncrease(DAMAGE_BONUS_2, DAMAGE_TYPE_BLUDGEONING), eWarriorBuff);
    if (GetHitDice(oNPC) >= 8) iKeen = ItemPropertyKeen();
    if (GetHitDice(oNPC) >= 21) eWarriorBuff = EffectLinkEffects(EffectAttackIncrease(2), eWarriorBuff);
    if (GetHitDice(oNPC) >= 24) eWarriorBuff = EffectLinkEffects(EffectDamageIncrease(DAMAGE_BONUS_4, DAMAGE_TYPE_BLUDGEONING), eWarriorBuff);
    if (GetHitDice(oNPC) >= 28) iMassive = ItemPropertyMassiveCritical(IP_CONST_DAMAGEBONUS_2d6);
    eWarriorBuff = SupernaturalEffect(eWarriorBuff);
    if (GetHitDice(oNPC) > 3) ApplyEffectToObject(DURATION_TYPE_PERMANENT, eWarriorBuff, oNPC);

    // Execute default OnSpawn script.
    if(GetLocalInt(GetModule(), "X4_VANILLA_AI") == TRUE) SetLocalInt(OBJECT_SELF, "X4_VANILLA_AI", TRUE);
    if(GetLocalInt(OBJECT_SELF, "X4_VANILLA_AI") == TRUE) ExecuteScript("nw_c2_default9_c", OBJECT_SELF);
    else ExecuteScript("nw_c2_default9", OBJECT_SELF);


    //Post Spawn event requeste
    if (nSpecEvent == 2 || nSpecEvent == 3)
    {
    SignalEvent(OBJECT_SELF,EventUserDefined(EVENT_USER_DEFINED_POSTSPAWN));
    }

}
