//::///////////////////////////////////////////////
//:: Example XP2 OnItemEquipped
//:: x4_mod_def_equ
//:: (c) 2003 Bioware Corp.
//:://////////////////////////////////////////////
/*
    Put into: OnEquip Event
*/
//:://////////////////////////////////////////////
//:: Created By: Georg Zoeller
//:: Created On: 2003-07-16
//:://////////////////////////////////////////////
//:: Modified By: Deva Winblood
//:: Modified On: April 15th, 2008
//:: Added Support for Mounted Archery Feat penalties
//:://////////////////////////////////////////////

#include "x2_inc_switches"
#include "x2_inc_intweapon"
#include "x3_inc_horse"
//#include "70_inc_itemprop"
//#include "prc_x2_itemprop"
#include "x4_inc_functions"

const int IP_USE_LIMITATION_GENDER = 150;
void ZEPGenderRestrict(object oItem, object oPC)
{
// ---------------------------------------------------
    // CEP Gender restriction property script
    // Searches the item just equipped for a gender
    // restriction property.  If it finds it, it will
    // check the PC's gender against the appropriate value
    // and de-equip the item if they are found not to
    // match.
    // ---------------------------------------------------
    //First we check if this has the item property: Use Limitation: Gender.
    //If so, we enter the if statment and check PC gender
    //vs. the item's limitation.  Else we continue out of the
    //function.
    if (GetItemHasItemProperty(oItem, IP_USE_LIMITATION_GENDER))
        {
        itemproperty ipGenderProperty=GetFirstItemProperty(oItem);
        //We're not sure if the above property is the one
        //we want, so we'll check it vs. the Gender property,
        //and, if it's not it, loop through until we find it.
        while ((GetIsItemPropertyValid(ipGenderProperty))&&(GetItemPropertyType(ipGenderProperty)!= IP_USE_LIMITATION_GENDER))
            {
            ipGenderProperty=GetNextItemProperty(oItem);
            }
        //If, after all that, the property is invalid for
        //some reason, we return.  Else we now have a property
        //with the data of the Gender restriction of teh PC's
        //item.
        if (!GetIsItemPropertyValid(ipGenderProperty)) return;
        //Next line is kind of long and wonky looking, but bear
        //with me as I'm doing this to cut out variables.
        //We're comparing the item property parameter value (gender
        //type) vs. the PCs.  Theoretically they use the same
        //scale...
        //If they are not the same, we de-euip the item
        if (GetItemPropertySubType(ipGenderProperty)!=GetGender(oPC))
            {
            //Not equal, so take it off!
            AssignCommand(oPC,ActionUnequipItem(oItem));
            //Tell PC why.
            SendMessageToPC(oPC,"Your gender can't equip that!");
            }
        }
}

void main()
{

    object oItem = GetPCItemLastEquipped();
    object oPC   = GetPCItemLastEquippedBy();
    AssignCommand(oPC, ClearAllActions());
    ZEPGenderRestrict(oItem, oPC);

/*     //Disallow unequipping items (aside from items held in hands) in combat if the X4_UNEQUIP_LIMIT module int is set to TRUE
    if(GetLocalInt(GetModule(), "X4_UNEQUIP_LIMIT") == TRUE && GetSlotByItem(oItem, oPC) != INVENTORY_SLOT_RIGHTHAND && GetSlotByItem(oItem, oPC) != INVENTORY_SLOT_LEFTHAND && GetSlotByItem(oItem, oPC) != INVENTORY_SLOT_HEAD && GetSlotByItem(oItem, oPC) != INVENTORY_SLOT_ARROWS && GetSlotByItem(oItem, oPC) != INVENTORY_SLOT_BULLETS && GetSlotByItem(oItem, oPC) != INVENTORY_SLOT_BOLTS && GetIsInCombat(oPC) == TRUE)
    {
        if (GetLocalInt(oPC, "Polymorphed") == TRUE) DelayCommand(1.0, DeleteLocalInt(oPC, "Polymorphed"));
        else {
        AssignCommand(oPC, ActionUnequipItem(oItem));
        SendMessageToPC(oPC, "You have no time to change your equipment right now!");
        return;
        }
    }
 */
    //Disallow equpping mystical items in dangerous areas (regular wilderness areas and dungeons)
    if (GetLocalString(GetArea(oPC), "AreaString") != "" && GetLocalInt(GetArea(oPC), "SafeRest") == FALSE && GetStringLeft(GetResRef(oItem), 10) == "anc_it_mys")
    {
        if (GetLocalInt(oPC, "Polymorphed") == TRUE) DelayCommand(1.0, DeleteLocalInt(oPC, "Polymorphed"));
        else {
        AssignCommand(oPC, ActionUnequipItem(oItem));
        SendMessageToPC(oPC, "You can put on mystical items only in safe areas!");
        return;
        }
    }

    //Designed to stop PCs from reequipping recently disarmed weapons - works only in conjunction with code in "x4_mod_def_unaqu" script
    if (GetLocalInt(oItem, "EQUIP_DELAY"))
    {
        AssignCommand(oPC, ActionUnequipItem(oItem));
        SendMessageToPC(oPC, "You must wait 3 rounds before equipping this weapon!");
        return;
    }

    // -------------------------------------------------------------------------
    // Intelligent Weapon System
    // -------------------------------------------------------------------------
    if (IPGetIsIntelligentWeapon(oItem))
    {
        IWSetIntelligentWeaponEquipped(oPC,oItem);
        // prevent players from reequipping their weapon in
        if (IWGetIsInIntelligentWeaponConversation(oPC))
        {
                object oConv =   GetLocalObject(oPC,"X2_O_INTWEAPON_SPIRIT");
                IWEndIntelligentWeaponConversation(oConv, oPC);
        }
        else
        {
            //------------------------------------------------------------------
            // Trigger Drain Health Event
            //------------------------------------------------------------------
            if (GetLocalInt(oPC,"X2_L_ENSERRIC_ASKED_Q3")==1)
            {
                ExecuteScript ("x2_ens_dodrain",oPC);
            }
            else
            {
                IWPlayRandomEquipComment(oPC,oItem);
            }
        }
    }

    // -------------------------------------------------------------------------
    // Mounted benefits control
    // -------------------------------------------------------------------------
    if (GetWeaponRanged(oItem))
    {
        SetLocalInt(oPC,"bX3_M_ARCHERY",TRUE);
        HORSE_SupportAdjustMountedArcheryPenalty(oPC);
    }

    // -------------------------------------------------------------------------
    // Generic Item Script Execution Code
    // If MODULE_SWITCH_EXECUTE_TAGBASED_SCRIPTS is set to TRUE on the module,
    // it will execute a script that has the same name as the item's tag
    // inside this script you can manage scripts for all events by checking against
    // GetUserDefinedItemEventNumber(). See x2_it_example.nss
    // -------------------------------------------------------------------------
    if (GetModuleSwitchValue(MODULE_SWITCH_ENABLE_TAGBASED_SCRIPTS) == TRUE)
    {
        SetUserDefinedItemEventNumber(X2_ITEM_EVENT_EQUIP);
        int nRet =   ExecuteScriptAndReturnInt(GetUserDefinedItemEventScriptName(oItem),OBJECT_SELF);
        if (nRet == X2_EXECUTE_SCRIPT_END)
        {
            return;
        }
    }

/*     //1.71: wounding item property handling
    if(GetItemHasItemProperty(oItem,69))
    {
        itemproperty ip = GetFirstItemProperty(oItem);
        while(GetIsItemPropertyValid(ip))
        {
            if(GetItemPropertyType(ip) == 69)
            {
                ApplyWounding(oItem,oPC,GetItemPropertyCostTableValue(ip));
            }
            ip = GetNextItemProperty(oItem);
        }
    } */

    //Remove all effects releated with epic superior weapon focus (damage decrease)
        effect e = GetFirstEffect(oPC);
        while(GetIsEffectValid(e))//loop all effects
            {
                if(GetEffectCreator(e) == GetModule() && GetEffectType(e) == EFFECT_TYPE_ATTACK_INCREASE)
                {
                    RemoveEffect(oPC,e);//remove it
                }
                e = GetNextEffect(oPC);
            }
    //Apply Epic Superior Weapon Focus attack nerf - the bonus is now +1 at level 13, +2 at level 21, +3 at level 30
        int nSuperiorAttack = 0;
        if (GetHasFeat(1119, oPC))
            {
            switch (GetLevelByClass(CLASS_TYPE_WEAPON_MASTER, oPC))
                {
                case 13:
                case 14:
                case 15:
                case 16:
                case 17:
                case 18:
                case 19:
                case 20: nSuperiorAttack = 1; break;
                case 21:
                case 22:
                case 23:
                case 24:
                case 25:
                case 26:
                case 27:
                case 28:
                case 29: nSuperiorAttack = 2; break;
                case 30: nSuperiorAttack = 3; break;
                }
            effect eSuperiorAttack;
            int nWoC = GetHasWeaponOfChoiceEquipped(oPC);
            if (nWoC == 1) eSuperiorAttack = EffectAttackIncrease(nSuperiorAttack, ATTACK_BONUS_ONHAND);
            if (nWoC == 2) eSuperiorAttack = EffectAttackIncrease(nSuperiorAttack, ATTACK_BONUS_OFFHAND);
            if (nWoC == 3) eSuperiorAttack = EffectAttackIncrease(nSuperiorAttack, ATTACK_BONUS_MISC);
            eSuperiorAttack = SupernaturalEffect(eSuperiorAttack);
            if (nSuperiorAttack != 0 && nWoC != 0) AssignCommand(GetModule(), ApplyEffectToPCAndHideIcon(eSuperiorAttack, oPC));
            }

    {
     //Put your custom code here.
    }

}
