//::///////////////////////////////////////////////
//:: Name: x4_nar_mod_def_rest
//:: Copyright (c) 2014 Bioware Corp.
//:://////////////////////////////////////////////
/*
    The generic resting system, including wandering
    monsters and food requirement
*/
//:://////////////////////////////////////////////
//:: Created By: Georg Zoeller
//:: Created On: June 9/03
//:://////////////////////////////////////////////
//:: Modified By: Deva Winblood
//:: Modified Date: January 28th, 2008
//:://////////////////////////////////////////////

#include "x2_inc_restsys"
#include "x2_inc_switches"
#include "x3_inc_horse"
#include "events_inc"

void WanderingMonsterSpawn(object oPC)
{
    //The function fires twice normally - we need to get around this
    if (GetLocalInt(oPC, "WanderingSpawned") == TRUE) return;
    SetLocalInt(oPC, "WanderingSpawned", TRUE);
    DelayCommand(1.0, DeleteLocalInt(oPC, "WanderingSpawned"));
    //First let's halt the function if there's someone else resting in the area
    object oObject = GetFirstObjectInArea(GetArea(oPC));
    while (GetIsObjectValid(oObject))
    {
        if (GetIsResting(oObject) == TRUE && oObject != oPC) return;
        oObject = GetNextObjectInArea(GetArea(oPC));
    }

    //Now let's get right to it!
    string sAreaString = GetLocalString(GetArea(oPC), "AreaString");
    string sRegionString = GetStringLeft(sAreaString, 1);
    if (sRegionString == "" || GetLocalInt(GetArea(oPC), "SafeRest") == TRUE) return;
    int nLevel = GetHitDice(oPC)-1;
    oObject = GetFirstObjectInArea(GetArea(oPC));
    while (GetIsObjectValid(oObject))
    {
        if (oObject != oPC && GetIsPC(oObject))
        {
            nLevel = nLevel+1;
            break;
        }
        oObject = GetNextObjectInArea(GetArea(oPC));
    }
    SelectEnemies(sRegionString, nLevel, oPC, ENCOUNTER_TYPE_NORMAL);
    //Select possible enemies
    string sEnemy1 = GetLocalString(oPC, "sEnemy1");
    string sEnemy2 = GetLocalString(oPC, "sEnemy2");
    string sEnemy3 = GetLocalString(oPC, "sEnemy3");
    string sEnemy4 = GetLocalString(oPC, "sEnemy4");
    string sEnemy5 = GetLocalString(oPC, "sEnemy5");
    string sEnemy6 = GetLocalString(oPC, "sEnemy6");
    DeleteEnemyVar(oPC);
    object oCreature = CreateObject(OBJECT_TYPE_CREATURE, SelectMain(sEnemy1, sEnemy2, sEnemy3, sEnemy4, sEnemy5, sEnemy6), GetLocation(oPC));
    ScaleCreature(oCreature, nLevel);
    AssignCommand(oPC, ClearAllActions());
    FloatingTextStringOnCreature("Your rest is interrupted by an enemy!", oPC, FALSE);
}

void main()
{
    object oPC = GetLastPCRested();
    object oMount;

    //Let's allow the PC to use a sequencer again
    if (GetLastRestEventType() == REST_EVENTTYPE_REST_FINISHED) DeleteLocalInt(oPC, "SequencerUsed");

    //We want the PCs to be unable to use area transition for a few seconds (12) after attempting resting (this is to avoid people quickly escaping the area if a monster appears)
    if (GetLastRestEventType() == REST_EVENTTYPE_REST_STARTED)
    {
        SetLocalInt(oPC, "JustRested", TRUE);
        DelayCommand(12.0, DeleteLocalInt(oPC, "JustRested"));
    }

    //We want to save characters in ONLINE mode on rest, in case of a server crash - the delay is to ensure the PC is not polymorphed
    if (GetLocalString(GetModule(), "MODE") == "ONLINE" && GetLastRestEventType() == REST_EVENTTYPE_REST_FINISHED) DelayCommand(1.0, ExportSingleCharacter(oPC));

    {
    // Put custom code here
    }

    if (!GetLocalInt(GetModule(),"X3_MOUNT_NO_REST_DISMOUNT"))
    { // make sure not mounted
        /*  Deva, Jan 17, 2008
            Do not allow a mounted PC to rest
        */
        if (HorseGetIsMounted(oPC))
        { // cannot mount
            if (GetLocalInt(oPC,"X3_REST_CANCEL_MESSAGE_SENT"))
            { // cancel message already played
                DeleteLocalInt(oPC,"X3_REST_CANCEL_MESSAGE_SENT");
            } // cancel message already played
            else
            { // play cancel message
                FloatingTextStrRefOnCreature(112006,oPC,FALSE);
                SetLocalInt(oPC,"X3_REST_CANCEL_MESSAGE_SENT",TRUE); // sentinel
                // value to prevent message played a 2nd time on canceled rest
            } // play cancel message
            AssignCommand(oPC,ClearAllActions(TRUE));
            return;
        } // cannot mount
    } // make sure not mounted

    if (!GetLocalInt(GetModule(),"X3_MOUNT_NO_REST_DESPAWN"))
    { // if there is a paladin mount despawn it
        oMount=HorseGetPaladinMount(oPC);
        if (!GetIsObjectValid(oMount)) oMount=GetLocalObject(oPC,"oX3PaladinMount");
        if (GetIsObjectValid(oMount))
        { // paladin mount exists
            if (oMount==oPC||!GetIsObjectValid(GetMaster(oMount))) AssignCommand(oPC,HorseUnsummonPaladinMount());
            else { AssignCommand(GetMaster(oMount),HorseUnsummonPaladinMount()); }
        } // paladin mount exists
    } // if there is a paladin mount despawn it

    //A 25% chance of an enemy interrupting the rest - should not happen if there's already somebody resting in the area
    if (Random(4) == 0 && GetLastRestEventType() == REST_EVENTTYPE_REST_STARTED) WanderingMonsterSpawn(oPC);

    if (GetModuleSwitchValue(MODULE_SWITCH_USE_XP2_RESTSYSTEM) == TRUE)
    {
        /*  Georg, August 11, 2003
            Added this code to allow the designer to specify a variable on the module
            Instead of using a OnAreaEnter script. Nice new toolset feature!
            Basically, the first time a player rests, the area is scanned for the
            encounter table string and will set it up.
        */
        object oArea = GetArea (oPC);

        string sTable = GetLocalString(oArea,"X2_WM_ENCOUNTERTABLE") ;
        if (sTable != "" )
        {
            int nDoors = GetLocalInt(oArea,"X2_WM_AREA_USEDOORS");
            int nDC = GetLocalInt(oArea,"X2_WM_AREA_LISTENCHECK");
            WMSetAreaTable(oArea,sTable,nDoors,nDC);

            //remove string to indicate we are set up
            DeleteLocalString(oArea,"X2_WM_ENCOUNTERTABLE");
        }


        /* Brent, July 2 2003
           - If you rest and are a low level character at the beginning of the module.
             You will trigger the first dream cutscene
        */
        if (GetLocalInt(GetModule(), "X2_G_LOWLEVELSTART") == 10)
        {
            AssignCommand(oPC, ClearAllActions());
            if (GetHitDice(oPC) >= 12)
            {
                ExecuteScript("bk_sleep", oPC);
                return;
            }
            else
            {
                FloatingTextStrRefOnCreature(84141 , oPC);
                return;
            }
        }

        if (GetLastRestEventType()==REST_EVENTTYPE_REST_STARTED)
        {
            if (!WMStartPlayerRest(oPC))
            {
                // The resting system has objections against resting here and now
                // Probably because there is an ambush already in progress
                FloatingTextStrRefOnCreature(84142  ,oPC);
                AssignCommand(oPC,ClearAllActions());
            }
            if (WMCheckForWanderingMonster(oPC))
            {
                //This script MUST be run or the player won't be able to rest again ...
                ExecuteScript("x2_restsys_ambus",oPC);
            }
        }
        else if (GetLastRestEventType()==REST_EVENTTYPE_REST_CANCELLED)
        {
         // No longer used but left in for the community
         // WMFinishPlayerRest(oPC,TRUE); // removes sleep effect, etc
        }
        else if (GetLastRestEventType()==REST_EVENTTYPE_REST_FINISHED)
        {
         // No longer used but left in for the community
         //   WMFinishPlayerRest(oPC); // removes sleep effect, etc
        }
    }
}

