//::///////////////////////////////////////////////
//:: Isaacs Greater Missile Storm
//:: x0_s0_MissStorm2
//:: Copyright (c) 2002 Bioware Corp.
//:://////////////////////////////////////////////
/*
 Up to 20 missiles, each doing 3d6 damage to each
 target in area.
*/
//:://////////////////////////////////////////////
//:: Created By: Brent
//:: Created On: July 31, 2002
//:://////////////////////////////////////////////
//:: Last Updated By:

#include "70_inc_spells"
#include "x0_i0_spells"
#include "x2_inc_spellhook"

//This version of the spell applies all damage as a cumulated single hit instead of spreading it among all the missiles.
//This has been done because before that the spell used to be useless against any target with 15 magical resistance or more.
void DoMassMissileStorm(int nD6Dice, int nCap, int nSpell, int nMIRV = VFX_IMP_MIRV, int nVIS = VFX_IMP_MAGBLUE, int nDAMAGETYPE = DAMAGE_TYPE_MAGICAL, int nONEHIT = FALSE, int nReflexSave = FALSE)
{
    if(spell.Caster == OBJECT_INVALID) spellsDeclareMajorVariables();//pre 1.70 spellscripts support
    int nCasterLvl = spell.Level;
    int i, nSAVETYPE, nCnt = 1;
    effect eMissile = EffectVisualEffect(nMIRV);
    effect eVis = EffectVisualEffect(nVIS);
    float fDist = 0.0;
    float fDelay = 0.0;
    float fDelay2, fTime;
    int nMissiles = nCasterLvl;

    if (nMissiles > nCap)
    {
        nMissiles = nCap;
    }

        /* New Algorithm
            1. Count # of targets
            2. Determine number of missiles
            3. First target gets a missile and all Excess missiles
            4. Rest of targets (max nMissiles) get one missile
       */
    int nEnemies = 0;

    object oTarget = GetFirstObjectInShape(SHAPE_SPHERE, RADIUS_SIZE_GARGANTUAN, spell.Loc, TRUE, OBJECT_TYPE_CREATURE);
    //Cycle through the targets within the spell shape until an invalid object is captured.
    while (GetIsObjectValid(oTarget))
    {
        // * caster cannot be harmed by this spell
        if (oTarget != spell.Caster && spellsIsTarget(oTarget, SPELL_TARGET_SELECTIVEHOSTILE, spell.Caster))
        {
            // GZ: You can only fire missiles on visible targets
            // If the firing object is a placeable (such as a projectile trap),
            // we skip the line of sight check as placeables can't "see" things.
            if ( ( GetObjectType(spell.Caster) == OBJECT_TYPE_PLACEABLE ) ||
                   GetObjectSeen(oTarget,spell.Caster))
            {
                nEnemies++;
            }
        }
        oTarget = GetNextObjectInShape(SHAPE_SPHERE, RADIUS_SIZE_GARGANTUAN, spell.Loc, TRUE, OBJECT_TYPE_CREATURE);
     }

     if (nEnemies == 0) return; // * Exit if no enemies to hit
     int nExtraMissiles = nMissiles / nEnemies;

     // April 2003
     // * if more enemies than missiles, need to make sure that at least
     // * one missile will hit each of the enemies
     if (nExtraMissiles <= 0)
     {
        nExtraMissiles = 1;
     }

     switch(nDAMAGETYPE)//1.71: support for custom content missile storm spells with nonstandard damage type
     {
     case DAMAGE_TYPE_ACID:
     nSAVETYPE = SAVING_THROW_TYPE_ACID;
     break;
     case DAMAGE_TYPE_ELECTRICAL:
     nSAVETYPE = SAVING_THROW_TYPE_ELECTRICITY;
     break;
     case DAMAGE_TYPE_FIRE:
     nSAVETYPE = SAVING_THROW_TYPE_FIRE;
     break;
     case DAMAGE_TYPE_COLD:
     nSAVETYPE = SAVING_THROW_TYPE_COLD;
     break;
     case DAMAGE_TYPE_SONIC:
     nSAVETYPE = SAVING_THROW_TYPE_SONIC;
     break;
     case DAMAGE_TYPE_POSITIVE:
     nSAVETYPE = SAVING_THROW_TYPE_POSITIVE;
     break;
     case DAMAGE_TYPE_NEGATIVE:
     nSAVETYPE = SAVING_THROW_TYPE_NEGATIVE;
     break;
     }

     // by default the Remainder will be 0 (if more than enough enemies for all the missiles)
     int nRemainder = 0;

     if (nExtraMissiles >0)
        nRemainder = nMissiles % nEnemies;

     if (nEnemies > nMissiles)
        nEnemies = nMissiles;

    oTarget = GetFirstObjectInShape(SHAPE_SPHERE, RADIUS_SIZE_GARGANTUAN, spell.Loc, TRUE, OBJECT_TYPE_CREATURE);
    if (nSpell == SPELL_BALL_LIGHTNING)//1.71: ball lightning has a single target area of effect
    {
        oTarget = spell.Target;
        nEnemies = 1;
        nExtraMissiles = nMissiles;
        nRemainder = 0;
    }
    //Cycle through the targets within the spell shape until an invalid object is captured.
    while (GetIsObjectValid(oTarget) && nCnt <= nEnemies)
    {
        // * caster cannot be harmed by this spell
        if (oTarget != spell.Caster && spellsIsTarget(oTarget, SPELL_TARGET_SELECTIVEHOSTILE, spell.Caster) &&
           (GetObjectType(spell.Caster) == OBJECT_TYPE_PLACEABLE || GetObjectSeen(oTarget,spell.Caster)))
        {
                //Fire cast spell at event for the specified target
                SignalEvent(oTarget, EventSpellCastAt(spell.Caster, nSpell));

                // * recalculate appropriate distances
                fDist = GetDistanceBetween(spell.Caster, oTarget);
                fDelay = fDist/(3.0 * log(fDist) + 2.0);

                // Firebrand.
                // It means that once the target has taken damage this round from the
                // spell it won't take subsequent damage
                if (nONEHIT)
                {
                    nExtraMissiles = 1;
                    nRemainder = 0;
                }

                //--------------------------------------------------------------
                // GZ: Moved SR check out of loop to have 1 check per target
                //     not one check per missile, which would rip spell mantels
                //     apart
                //--------------------------------------------------------------
                if (!MyResistSpell(spell.Caster, oTarget, fDelay))
                {
                    nCap = nExtraMissiles + (nRemainder > 0);//1.71: this will distribute remainder missiles evenly amongs all targets
                    for (i=1; i <= nCap; i++)
                    {
                        //Roll damage
                        int nDam = MaximizeOrEmpower(6,nD6Dice,spell.Meta);
                        // if reflexsave allowed, make evasion check
                        if(nReflexSave)
                        {
                            nDam = GetReflexAdjustedDamage(nDam, oTarget, spell.DC, nSAVETYPE, spell.Caster);
                        }

                        fDelay2 += 0.1;
                        fTime = fDelay + fDelay2;

                        //Set damage effect
                        effect eDam = EffectDamage(nDam, nDAMAGETYPE);
                        //Apply the MIRV and damage effect
                        DelayCommand(fTime, ApplyEffectToObject(DURATION_TYPE_TEMPORARY, eVis, oTarget));
                        DelayCommand(fDelay2, ApplyEffectToObject(DURATION_TYPE_INSTANT, eMissile, oTarget));
                        //do not bother when no damage should happen anyway
                        SetLocalInt(oTarget, "msldmg", GetLocalInt(oTarget, "msldmg") + nDam );
                        if(i == (nExtraMissiles + nRemainder))
                        {
                         effect eDamage = EffectDamage(GetLocalInt(oTarget, "msldmg"), nDAMAGETYPE);
                         DelayCommand(fTime, ApplyEffectToObject(DURATION_TYPE_INSTANT, eDamage, oTarget));
                         DelayCommand(fTime, SetLocalInt(oTarget, "msldmg", 0));
                        }
                    }
                } // for
                else
                {   // * apply a dummy visual effect
                    ApplyEffectToObject(DURATION_TYPE_INSTANT, eMissile, oTarget);
                }
                nCnt++;// * increment count of missiles fired
                nRemainder--;
        }
        oTarget = GetNextObjectInShape(SHAPE_SPHERE, RADIUS_SIZE_GARGANTUAN, spell.Loc, TRUE, OBJECT_TYPE_CREATURE);
    }
}

void main()
{

/*
  Spellcast Hook Code
  Added 2003-06-20 by Georg
  If you want to make changes to all spells,
  check x2_inc_spellhook.nss to find out more

*/

    if (!X2PreSpellCastCode())
    {
    // If code within the PreSpellCastHook (i.e. UMD) reports FALSE, do not run this spell
        return;
    }

// End of Spell Cast Hook

    spellsDeclareMajorVariables();
   //SpawnScriptDebugger();
    DoMassMissileStorm(1, 15, spell.Id);
}
